from vector import *
from OpenGL.GL import *
from math import sin, cos, acos, asin, radians
from random import randint, seed
from time import time

PI = 3.14159265

class quad:
	def __init__(self, p1, p2, p3, p4, color = Vec3(0.7,0.7,0.7)):
		self.p1 = p1
		self.p2 = p2
		self.p3 = p3
		self.p4 = p4
		self.color = color

class Sphere:
	def __init__(self, segments, sides, diameter, actvseg = 0):
		self.sides  = sides
		self.dia    = diameter
		self.numseg = segments
		self.segments = [] # Each segments
		self.relevant = [] # The segments that are a part of the pattern
		self.genSegments() # Make our segments
		
		if actvseg == 0: self.actvseg = int(segments/2)
		else: self.actvseg = actvseg
		self.segments[self.actvseg].active = True
		
	def genSegments(self):
		v = radians(180/self.numseg)
		r = -self.dia/2
		# Make the basic segments
		for i in range(0, self.numseg):
			self.segments.append(sphereSegment(self.sides,
										 cos(v*i-(PI/2))*r,      # Radius for the upper ring
										 cos(v*(i+1)-(PI/2))*r,  # Radius for the lower ring
										 sin(v*i-(PI/2))*r,      # Y for the higher ring
										 sin(v*(i+1)-(PI/2))*r)) # Y for the lower ring)
			self.segments[i].genQuads()
			
	def draw(self):		
		for i in range(0, len(self.segments)):
			self.segments[i].draw()
			
	def next(self):
		self.segments[self.actvseg].active = False
		self.actvseg += 1
		if self.actvseg > len(self.segments)-2:
			self.actvseg = 1
		self.segments[self.actvseg].active = True	
	
	def prev(self):
		self.segments[self.actvseg].active = False
		self.actvseg -= 1
		if self.actvseg < 1:
			self.actvseg = len(self.segments) - 2
		self.segments[self.actvseg].active = True
		
	def rotate(self, segment, steps):
		self.segments[segment].rot += steps
		if self.segments[segment].rot >= self.sides:
			self.segments[segment].rot = self.segments[segment].rot % self.sides
		if self.segments[segment].rot < 0:
			self.segments[segment].rot = self.sides-self.segments[segment].rot-2

	def scramble(self):
		seed(time())
		for n in self.segments:
			n.rot = randint(0, self.sides)
			
class sphereSegment:
	def __init__(self, sides = 12, highrad = 5, lowrad = 5, highy = 1, lowy = 0):
		self.highr  = highrad	# Radius of the upper row of vertecies
		self.lowr   = lowrad	# Lower row...
		self.highy  = highy		# Y-position of the vertecies in the upper row
		self.lowy   = lowy		# Lower row again..
		self.sides  = sides		# Sides.. dug
		self.active = False		# Is this segment active for modification? 
		self.quads  = []		# List for the outward-facing sides		
		self.rot    = 0			# Rotation in "sides", so we can twist and stuff
		
	# Generate our segments outward facing polygons
	def genQuads(self):
		rad = -radians(360/self.sides)
		for i in range(0, self.sides):
			p1 = Vec3(cos(rad*i)    *self.highr, self.highy, sin(rad*i)    *self.highr)
			p2 = Vec3(cos(rad*(i+1))*self.highr, self.highy, sin(rad*(i+1))*self.highr)
			p3 = Vec3(cos(rad*(i+1))*self.lowr,  self.lowy,  sin(rad*(i+1))*self.lowr)
			p4 = Vec3(cos(rad*i)    *self.lowr,  self.lowy,  sin(rad*i)    *self.lowr)
			self.quads.append(quad(p1, p2, p3, p4))
		
	def draw(self):
		glRotatef(self.rot*(360/self.sides), 0, 1, 0)
		glBegin(GL_QUADS)
		for n in self.quads:
			if self.active:	glColor3f((n.color.x + 1) / 2, (n.color.y + 1) / 2, (n.color.z + 1) / 2)
			else:			glColor3f(n.color.x, n.color.y, n.color.z)
			glVertex3f(n.p1.x, n.p1.y, n.p1.z)
			glVertex3f(n.p2.x, n.p2.y, n.p2.z)
			glVertex3f(n.p3.x, n.p3.y, n.p3.z)
			glVertex3f(n.p4.x, n.p4.y, n.p4.z)
		glEnd()
		glRotatef(-self.rot*(360/self.sides), 0, 1, 0)
		#glColor3f(1,1,1)
